/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry
*
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#ifndef DOM_OUTPUT_H
#define DOM_OUTPUT_H

#include <string>

namespace domtools {

   
   class basic_output {
    public:
      static int doEncode;
      /// Do not do any entity encoding
      static const int ENCODE_NONE = 0;
      /// Do full entity encoding (all '&','<','>','"','\'' that are not part of markup)
      static const int ENCODE_FULL = 1;
      /// As ENCODE_FULL, but do not encode characters that are part of a valid entity already.
      static const int ENCODE_PARTIAL = 2;
      
      virtual ~basic_output()
	{
	}
      
      virtual void putTag(const std::string & tag, const std::string & cdata);

      virtual void beginTag(const std::string & tag) = 0;
      virtual void beginPI(const std::string & tag) = 0;
      virtual void putAttribute(const std::string &, const std::string &) = 0;
      virtual void putAttribute(const std::string &) = 0;
      virtual void putCDATA(const std::string &) = 0;
      virtual void endTag(const std::string & tag) = 0;
      virtual void endPI(void) = 0;
      
      virtual void setWhitespace(bool whitespace) = 0;
      
     };
   
   class xml_output : public basic_output {
    private:
      virtual void outputWhitespace(int num);

      virtual void outputText(const std::string & o) = 0;
      virtual void outputXML(const std::string & o) = 0;
      virtual void outputLF(void)
	{
	   outputXML(endl);
	   col = 0;
	}
      
      bool debug;
      int nest;
      bool hasContent;
      bool hasChildren;
      bool firsttag;
      int lastattrcol;
      int nestws;
      bool inPI;
      std::string endl;
    protected:
      int col;
      bool ws;
    public:
      xml_output (void)
	{
	   nestws = 2;
	   debug = false;
	   nest = 0;
	   col = 0;
	   lastattrcol = 0;
	   ws = true;
	   hasContent= false;
	   hasChildren = false;
	   firsttag = true;
	   inPI = false;
	   endl = "\n";
	}

      virtual ~xml_output(void)
	{
	}
      
      virtual void setDebug(bool d)
	{
	   debug = d;
	}
      
      virtual void setLF(const std::string & lf)
	{
	   endl = lf;
	}
      virtual void beginTag(const std::string & tag);
      virtual void beginPI(const std::string & tag);
      virtual void putAttribute(const std::string & key);
      virtual void putAttribute(const std::string & key, const std::string & value);
      virtual void putCDATA(const std::string & text);
      virtual void endTag(const std::string & tag);
      virtual void endPI(void);
      
      virtual void setWhitespace(bool inWhitespace)
	{
	   ws = inWhitespace;
	}
     };

   

   class xml_string_output : public xml_output 
     {
      private:
	std::string out;
	
	void calcCol(const std::string & o)
	  {
	     std::string::size_type opos = o.find_last_of('\n');
	     if (opos != std::string::npos) {
		col += o.length() - opos;
	     } else col += o.length();
	  }

	virtual void outputText(const std::string & o)
	  {
	     // FIXME: Enforce encoding
	     if (ws) calcCol(o);	     
	     out += o;
	  }
	
	virtual void outputXML(const std::string & o)
	  {
	     if (ws) calcCol(o);
	     out += o;
	  }

      public:
	virtual ~xml_string_output()
	  {
	  }
	
	void copyToString(std::string & o) const
	  {
	     o = out;
	  }

	std::string getString(void) const
	  {
	     return out;
	  }
     };
   
}


#endif
