/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry
*
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#include "dom_print.h"
#include "dom_tools.h"

//#define XDebug(__cmd__) cerr << __FILE__ "," << __LINE__ << ": " << __cmd__ << endl;
#define XDebug(__cmd__)

using namespace std;
using namespace domtools;

void domtools::DomPrint::putDOMTree(const dom_ptr node)
{
   if (node->isNull()) return;

   string name;
   string value;
   
   dom_ptr tmp;
   dom_namedmap_ptr attrs;
   int len, pos;
   string::size_type spos;
   
   bool pi = false;
   
   switch (node->getNodeType()) {
    case Node::DOCUMENT_NODE:
      tmp = dom_ptr(node->getFirstChild());
      while (!tmp->isNull()) {
	 putDOMTree(tmp);
	 tmp = dom_ptr(tmp->getNextSibling());
      }
      return;
    case Node::XML_DECL_NODE:
      XDebug("XML_DECL_NODE");
      out.beginPI("xml");
      value= node->getVersion();
      out.putAttribute("version",value);
      value= node->getEncoding();
      if (value != "") out.putAttribute("encoding",value);
      value= node->getStandalone();
      if (value != "") out.putAttribute("standalone",value);
      out.endPI();
      return;
    case Node::PROCESSING_INSTRUCTION_NODE:
      pi = true;
      XDebug("PI");
      
      //-fallthrough
    case Node::ELEMENT_NODE:
      name = node->getNodeName();
      
      if (pi) out.beginPI(name);
      else out.beginTag(name);
      
      attrs = dom_namedmap_ptr(node->getAttributes());
      
      len = attrs->getLength();
      pos = 0;
      while (pos < len) {
	 tmp = dom_ptr(attrs->item(pos));
	 putDOMTree(tmp);
	 pos++;
      }
      
      canIgnore=true;
      tmp = dom_ptr(node->getFirstChild());
      while (!tmp->isNull()) {
	 putDOMTree(tmp);
	 tmp = dom_ptr(tmp->getNextSibling());
      }

      if (pi) out.endPI();
      else out.endTag(name);
      canIgnore=true;
      
      return;
    case Node::CDATA_SECTION_NODE:
      // We dont give a damn about the separation of CDATA and text. We will escape anything
      // anyway.
    case Node::TEXT_NODE:
      value = node->getNodeValue();
      if (value != "") {
	 if (ignorews && canIgnore) {
	    spos = value.find_first_not_of(" \t\r\n");
	    if (spos != string::npos) out.putCDATA(value.substr(spos));
	 } else out.putCDATA(value);
      }
      return;
    case Node::ATTRIBUTE_NODE:
      // FIXME: Handle attributes without value
      out.putAttribute( node->getNodeName() , node->getNodeValue() );
      return;
    default:
      out.putCDATA("[UNKNOWN NODE TYPE]");
      XDebug("[UNKNOWN NODE TYPE]\n");
   };
}
