/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry
*
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#include "dom_tools.h"

#define XDebug(__cmd__) cerr << __FILE__ "," << __LINE__ << ": " << __cmd__ << endl;

#include <cstdio>

using namespace std;

string domtools::ltos(long i)
{
   char buf[30];
   sprintf(buf,"%ld",i);
   return buf;
};


#include <cstring>

long domtools::toLong(const DOMString s)
{
   char * t = s.transcode();
   if (!t) return 0;
   long tmp = atol(t);
   delete [] t;
   return tmp;
}

ostream& domtools::operator<< (ostream& target, const DOMString s)
{
   char *p = s.transcode();
   if (!p) return target;
   target << p;
   delete [] p;
   return target;
}

string domtools::toString(const DOMString s)
{
   char * t = s.transcode();
   if (!t) return "";
   string tmp = t;
   delete [] t;
   return tmp;
}


/// Get a tag we know there should only be one of.
DOM_Node domtools::getSingleTag(const DOM_Document doc, const string & tag)
{
   DOM_NULL_REFERENCE(doc);
   const DOM_Node root = getFirstChildElement(doc);
   if (root.isNull()) return root;
   
   const DOMString ns = root.getNamespaceURI();
   const DOMString tmp(tag.c_str());
   DOM_NodeList nl = doc.getElementsByTagNameNS(ns,tmp);
   //	if (nl.getLength() != 1) return DOM_Node(0);
   return nl.item(0);
}


DOM_Node domtools::getSingleTag(const DOM_Node node, const string & tag)
{
   DOM_NULL_REFERENCE(node);
   if (node.getNodeType() == DOM_Node::DOCUMENT_NODE) return getSingleTag((const DOM_Document &)node,tag);
   const DOM_Document doc = node.getOwnerDocument();
   DOM_NULL_REFERENCE(doc);
   
   return getSingleTag(doc,tag);
}



   
void domtools::getNodeData(const DOM_Node node, string & out)
{
   out = "";
   DOM_NULL_REFERENCE(node);
   const DOM_NodeList n = node.getChildNodes();
   
   int i = 0;
   int len = n.getLength();
   while (i<len) {
      out += toString(n.item(i++).getNodeValue());
   }
}

void domtools::getFirstChildElementData(const DOM_Node node, string & out)
{
   out = "";
   DOM_NULL_REFERENCE(node);
   const DOM_Node n = getFirstChildElement(node);
   DOM_NULL_REFERENCE(n);
   getNodeData(n,out);
}

void domtools::getSingleTagData(const DOM_Document doc, const string & tag, string & out)
{
   const DOM_Node n = getSingleTag(doc,tag);
   if (n.isNull()) out = "";
   else getNodeData(n,out);
}      



DOM_Node domtools::getNextSiblingElement(const DOM_Node node)
{
   DOM_NULL_REFERENCE(node);
   DOM_Node tmp = node.getNextSibling();
   while (!tmp.isNull() && tmp.getNodeType() != DOM_Node::ELEMENT_NODE) tmp = tmp.getNextSibling();
   return tmp;
}



DOM_Node domtools::getFirstChildElement(const DOM_Node node)
{
   DOM_NULL_REFERENCE(node);
   DOM_Node tmp = node.getFirstChild();
   if (tmp.isNull() || tmp.getNodeType() == DOM_Node::ELEMENT_NODE) return tmp;
   return getNextSiblingElement(tmp);
}



DOM_Node domtools::getFirstChildOfTag(const DOM_Node node, const string & tag)
{
   DOM_NULL_REFERENCE(node);
   const DOM_Node tmp = getSingleTag(node,tag);
   DOM_NULL_REFERENCE(tmp);
   return getFirstChildElement(tmp);
}


DOM_Node domtools::getFirstChildOfTag(const DOM_Document node, const string & tag)
{
   DOM_NULL_REFERENCE(node);
   DOM_Node tmp = getSingleTag(node,tag);
   DOM_NULL_REFERENCE(tmp);
   return getFirstChildElement(tmp);
}



void domtools::getAttribute(const DOM_Node node, const string & attr, string & out)
{
   DOM_NULL_REFERENCE(node);
   DOM_NamedNodeMap map = node.getAttributes();
   out = "";
   
   DOM_Node a = map.getNamedItem(DOMString(attr.c_str()));
   if (a.isNull()) return;
   getNodeData(a,out);
}

void domtools::getAttribute(const DOM_Document node, const string & attr, string & out)
{
   DOM_NULL_REFERENCE(node);
   getAttribute((const DOM_Node &)node,attr,out);
}

void domtools::getAttribute(const DOM_Node node, const string & tag, const string & attr, string & out)
{
   DOM_NULL_REFERENCE(node);
   DOM_Node tmp = getSingleTag(node,tag);
   DOM_NULL_REFERENCE(tmp);
   getAttribute(tmp, attr,out);
}

void domtools::getAttribute(const DOM_Document node, const string & tag, const string & attr, string & out)
{
   DOM_NULL_REFERENCE(node);
   DOM_Node tmp = getSingleTag(node,tag);
   DOM_NULL_REFERENCE(tmp);
   getAttribute(tmp, attr,out);
}
