/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry
*
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#ifndef __DOM_TOOLS_H
#define __DOM_TOOLS_H

#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/DOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/XMLUniDefs.hpp>

#include "exceptions.h"

/// \file Most of hese functions are Xerces dependent, and are depreceated - Please look at dom_wrapper.h instead

namespace domtools {
   
   std::string ltos(long i);
   long toLong(const DOMString s);
   std::ostream& operator<< (std::ostream& target, const DOMString s);
   std::string toString(const DOMString s);
   
   /// Get a tag we know there should only be one of.
   DOM_Node getSingleTag(const DOM_Document doc, const std::string & tag);
   DOM_Node getSingleTag(const DOM_Node node, const std::string & tag);

   void getAttribute(const DOM_Node node, const std::string & attr, std::string & out);
   void getAttribute(const DOM_Document node, const std::string & attr, std::string & out);
   void getAttribute(const DOM_Node node, const std::string & tag, const std::string & attr, std::string & out);
   void getAttribute(const DOM_Document node, const std::string & tag, const std::string & attr, std::string & out);
   
   void getNodeData(const DOM_Node node, std::string & out);
   void getSingleTagData(const DOM_Document doc, const std::string & tag, std::string & out);
   void getFirstChildElementData(const DOM_Node node, std::string & out);

   
   DOM_Node getNextSiblingElement(const DOM_Node node);
   DOM_Node getFirstChildElement(const DOM_Node node);
   DOM_Node getFirstChildOfTag(const DOM_Node node, const std::string & tag);
   DOM_Node getFirstChildOfTag(const DOM_Document node, const std::string & tag);
}

//lint -e717
#define DOMTOOLS_EXCEPTION(__msg__) do { throw(domtools::Exception(__FILE__,__LINE__,__msg__)); } while(0)
#define DOM_NULL_REFERENCE(__n__) do { if ((__n__).isNull()) { DOMTOOLS_EXCEPTION("DOM_NULL_REFERENCE"); }  } while(0)
   

#endif
