/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry
*
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#include "dom_wrapper.h"

using namespace std;

string domtools::GenericNode::getFirstChildElementData(void)
{
   dom_ptr ptr(getFirstChildElement());
   if (ptr->isNull()) return "";
   return ptr->getNodeData();
}

string domtools::GenericNode::getTagData(const string & tag)
{
   dom_ptr ptr(getTag(tag));
   if (ptr->isNull()) return "";
   return ptr->getNodeData();
}


domtools::Node * domtools::Parser::parseString(const string & str)
{
   return parseMemory(str.c_str(),(int)str.length());
}


#ifdef HAVE_XERCES

domtools::Node * domtools::XercesNamedNodeMap::item(int pos)
{
   return new XercesNode(node.item(pos));
}
      
int domtools::XercesNamedNodeMap::getLength(void)
{
   return node.getLength();
}
      
bool domtools::XercesNode::isNull(void)
{
   return node.isNull();
}

string domtools::XercesNode::getLocalName(void)
{
   return domtools::toString(node.getLocalName());
}

unsigned short domtools::XercesNode::getNodeType(void)
{
   switch (node.getNodeType()) {
    case DOM_Node::DOCUMENT_NODE: return Node::DOCUMENT_NODE;
    case DOM_Node::ELEMENT_NODE: return Node::ELEMENT_NODE;
    case DOM_Node::XML_DECL_NODE: return Node::XML_DECL_NODE;
    case DOM_Node::PROCESSING_INSTRUCTION_NODE: return Node::PROCESSING_INSTRUCTION_NODE;
    case DOM_Node::CDATA_SECTION_NODE: return Node::CDATA_SECTION_NODE;
    case DOM_Node::TEXT_NODE: return Node::TEXT_NODE;
    case DOM_Node::ATTRIBUTE_NODE: return Node::ATTRIBUTE_NODE;
    default: DOMTOOLS_EXCEPTION("Unknown node type");
   };
}
      
domtools::Node * domtools::XercesParser::parseFile(const string & filename)
{
   ::DOMParser parser;
   parser.setDoNamespaces(true);
   parser.setToCreateXMLDeclTypeNode(false);
   try {
      parser.parse(filename.c_str());
      DOM_Document doc = parser.getDocument();
      if (doc.isNull()) return new XercesNode();
      return new XercesNode(doc);
   } catch(...) {
   }
   return new XercesNode();
}


domtools::Node * domtools::XercesParser::parseMemory(const void * const buffer, int length)
{
   ::DOMParser parser;
   parser.setDoNamespaces(true);
   parser.setToCreateXMLDeclTypeNode(false);
   MemBufInputSource* memBufIS = new MemBufInputSource
     (
      (const XMLByte*)buffer
      , length
      , "domtools"
      , false
      );
   
   try {
      parser.parse(*memBufIS);
      DOM_Document doc = parser.getDocument();
      delete memBufIS;
      if (!doc.isNull()) return new XercesNode(doc);
   } catch(...) {
      delete memBufIS;
   };
   return new XercesNode();
}


#endif
